/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include "CameraSetting.h"

#include "controllers/common/family/Families.h"
#include <boost/lexical_cast.hpp>

using namespace boost;

namespace camera {

const std::string CameraSetting::TYPE = "Camerasetting";
 const std::string CameraSetting:: BINNING[5] = {"NONE", "2048*2048","1024*1024","NONE","512*512"};
 const std::string CameraSetting:: MODE[5] = {"ND", "Single","ND","Multi","ND"};

CameraSetting::CameraSetting(const string& name) :
	ExperimentController(name), controller::Start(this) {

	setFamily(family::ACQUISITION, family::SETTING);

	binning.init(this, SAVE , "actualBinning", "wantedBinning");
	hflip.init(this, SAVE, "hflip");
	vflip.init(this, SAVE, "vflip");
	rotate.init(this, SAVE, "rotate");
	mode.init(this, SAVE, "actualmode", "wantedmode");
	nbKine.init(this, SAVE, "nbKine");
	kineTime.init(this, SAVE, "kineTime");
	stringbinning.init(this, NOSAVE| SPY, "actualstringbin", "wantedstringbin", "Binn");
	stringmode.init(this, NOSAVE| SPY, "actualstringmode", "wantedstringmode",  "Frame");
	temperature.init(this, SAVE| SPY, "actualtemp", "wantedtemp", "Temp");
	temperature.setUnit("~degree");

	m_CamDriver.init(this, "cammeraDriver");
	detectorController.init(this, "detectorController");
	detectorElement.init(this, "detectorElement");



	registerFunction(TYPE);




}

CameraSetting::~CameraSetting() {
}

void CameraSetting::postConfiguration() {

	binning=m_CamDriver->bind();
	mode=m_CamDriver->acqmode();
	temperature=m_CamDriver->actualTemperature();
	stringbinning=BINNING[binning()];
	stringmode=MODE[mode()];

	registerRefresher(binning, &CameraSetting::refreshBinning, this);
	registerRefresher(mode, &CameraSetting::refreshMode, this);
	registerUpdater(m_CamDriver->bind, &CameraSetting::updateBinning, this);
	registerUpdater(m_CamDriver->acqmode, &CameraSetting::updateMode, this);
	registerUpdater(m_CamDriver->actualTemperature, &CameraSetting::updateTemperature, this);

}

void CameraSetting::start() {




	int value=binning.setpoint();
	m_CamDriver->bind = value;
	m_CamDriver->acqmode=mode.setpoint();
	m_CamDriver->hflip = hflip();
	m_CamDriver->vflip = vflip();
	m_CamDriver->nbSlicesAcq = nbKine();
	//m_CamDriver->time = kineTime();
	if (value!=0){


	detectorController->detSize=(MAX_XSIZE/value *MAX_YSIZE/value);
	//detectorController->detSize=(MAX_XSIZE *MAX_YSIZE);
	detectorElement->xSize=(MAX_XSIZE/value);
	detectorElement->ySize=(MAX_YSIZE/value);
	}

	if (mode()==3){
		if (value!=0){


		detectorElement->wSize=nbKine();
		detectorController->detSize=(MAX_XSIZE/value *MAX_YSIZE/value)*nbKine();
		detectorController->wSize=nbKine();
	}
	}







	m_CamDriver->execute("writeParam");



}


void CameraSetting::refreshBinning(int32 value) {


}

void CameraSetting::refreshMode(int32 value) {


}

void CameraSetting::updateBinning() {
	binning=m_CamDriver->bind();
	stringbinning=BINNING[binning()];

}

void CameraSetting::updateTemperature() {
	temperature=m_CamDriver->actualTemperature();

}

void CameraSetting::updateMode() {

	mode=m_CamDriver->acqmode();
	stringmode=MODE[mode()];

}

}

