/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REALSTEREOELECDRIVER_H
#define REALSTEREOELECDRIVER_H

#include "StereoElecState.h"
#include "Utilities/Counter.h"
#include "TstereoDaqLib.h"
#include <QCoreApplication>
#include <common/base/Queue.h>
#include <boost/thread.hpp>
#include "Tstereo_ipmi_lib.h"

namespace stereo_elec {

class StereoElecDriver;

/**
 * Real implementation
 */
class RealStereoElecDriver: public StereoElecState {

public:

	/**
	 * Constructor.
	 *
	 * @param Link to the class which used the state
	 */
	RealStereoElecDriver(StereoElecDriver* owner);

	/**
	 * Destructor.
	 * Empty.
	 */
	virtual ~RealStereoElecDriver();

	/**
	 * Perform a init.
	 */
	virtual void init();
	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear();

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos();

	/*!
	 * \brief Stop command implementation
	 */
	virtual void start();
	/*!
	 * \brief Stop command implementation
	 */
	virtual void stop();

	/*!
	 * \brief Read command implementation
	 */
	virtual void read();

	/*!
	 * \brief Synchronize Read command implementation
	 */
	virtual void synchroniseRead();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus();

	virtual void config();

	virtual void readConfig();

	virtual void readMonitoring();


private:
	void qtLoop();
	void mtcaLoop();
    static const int32 READ_COMMAND;
    static const int32 START_COMMAND;
    static const int32 STOP_COMMAND;
    static const int32 SHUTDOWN_COMMAND;
    static const int32 READINFO_COMMAND;
    static const int32 CONFIG_COMMAND;
    static const int32 RESET_COMMAND;
    static const int32 READMTCA_COMMAND;
    common::Queue<int32> * m_commandQueue;
    boost::thread * m_serverThread;
    common::Queue<int32> * m_mtcaQueue;
    boost::thread * m_serverThread2;

	Counter m_TimeCounter; //! Time counter
	TstereoDaqLib *daq;
	Tstereo_ipmi_lib stereo_ipmi_lib;

	TmonFE8 monFE8[10];
	TmonLED monLED[2];
};

}
#endif //REALSTEREOELECDRIVER_H
